from django.apps import AppConfig
from django.core.checks import Tags, register
from django.utils.translation import gettext_lazy as _

from ambient_toolbox.static_role_permissions.settings import (
    get_static_role_permissions_enable_system_check,
    get_static_role_permissions_path,
)


class AmbientToolboxConfig(AppConfig):
    name = "ambient_toolbox"
    verbose_name = _("Ambient Toolbox")

    def ready(self):
        # System checks for static role permission feature
        if get_static_role_permissions_path() and get_static_role_permissions_enable_system_check():
            from ambient_toolbox.static_role_permissions.system_check import check_permissions_against_models

            register(check_permissions_against_models, Tags.models)



import sys
from unittest import mock

from django.core.management import call_command
from django.db import connection
from django.test import SimpleTestCase


class DetectGhostTableCommandTest(SimpleTestCase):
    @mock.patch.object(connection.introspection, "table_names", return_value=["invalid_table"])
    def test_management_command_match_found(self, *args):
        with mock.patch.object(sys.stdout, "write") as mocked_write:
            call_command("detect_ghost_tables")

        mocked_write.assert_called_with("* invalid_table\n")

    @mock.patch.object(connection.introspection, "table_names", return_value=[])
    def test_management_no_match_found(self, *args):
        with mock.patch.object(sys.stdout, "write") as mocked_write:
            call_command("detect_ghost_tables")

        mocked_write.assert_not_called()



# Generated by Django 5.0.7 on 2025-01-16 06:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("testapp", "0002_modelwithoutrelatednameonfieldandmeta_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="ModelWithGetOrNoneManagerModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("my_field", models.BooleanField(default=False)),
            ],
        ),
    ]



from django.db import models

from ambient_toolbox.managers import GetOrNoneManagerMixin


class ModelWithSelectorQuerySet(models.QuerySet):
    pass


class ModelWithGetOrNoneManager(GetOrNoneManagerMixin, models.Manager):
    pass



from django.contrib.auth.models import User
from django.test import TestCase

from ambient_toolbox.managers import (
    AbstractUserSpecificManager,
    AbstractUserSpecificQuerySet,
)
from testapp.models import ModelWithGetOrNoneManagerModel, MySingleSignalModel


class AbstractUserSpecificQuerySetTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.cqs = AbstractUserSpecificQuerySet()
        cls.user = User.objects.create(username="my-username")

    def test_default(self):
        self.assertEqual(self.cqs.default(self.user), self.cqs)

    def test_visible_for_regular(self):
        with self.assertRaisesMessage(NotImplementedError, "Please implement this method"):
            self.cqs.visible_for(self.user)

    def test_editable_for_regular(self):
        with self.assertRaisesMessage(NotImplementedError, "Please implement this method"):
            self.cqs.editable_for(self.user)

    def test_deletable_for_regular(self):
        with self.assertRaisesMessage(NotImplementedError, "Please implement this method"):
            self.cqs.deletable_for(self.user)


class TestUserSpecificManager(AbstractUserSpecificManager):
    pass


TestUserSpecificManager = TestUserSpecificManager.from_queryset(AbstractUserSpecificQuerySet)


class AbstractUserSpecificManagerTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.manager = TestUserSpecificManager()
        cls.user = User.objects.create(username="my-username")

    def test_visible_for_regular(self):
        with self.assertRaisesMessage(NotImplementedError, "Please implement this method"):
            self.manager.visible_for(self.user)

    def test_editable_for_regular(self):
        with self.assertRaisesMessage(NotImplementedError, "Please implement this method"):
            self.manager.editable_for(self.user)

    def test_deletable_for_regular(self):
        with self.assertRaisesMessage(NotImplementedError, "Please implement this method"):
            self.manager.deletable_for(self.user)


class GloballyVisibleQuerySetTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        # Create test user
        cls.user = User.objects.create(username="my-username")

        # Create list of objects
        cls.object_list = [
            MySingleSignalModel.objects.create(value=1),
            MySingleSignalModel.objects.create(value=2),
        ]

    def test_visible_for_regular(self):
        self.assertGreater(len(self.object_list), 0)
        self.assertEqual(
            MySingleSignalModel.objects.visible_for(self.user).count(),
            len(self.object_list),
        )

    def test_editable_for_regular(self):
        self.assertGreater(len(self.object_list), 0)
        self.assertEqual(
            MySingleSignalModel.objects.editable_for(self.user).count(),
            len(self.object_list),
        )

    def test_deletable_for_regular(self):
        self.assertGreater(len(self.object_list), 0)
        self.assertEqual(
            MySingleSignalModel.objects.deletable_for(self.user).count(),
            len(self.object_list),
        )


class GetOrNoneMixinTest(TestCase):
    def test_get_or_none_via_pk(self):
        new_obj = ModelWithGetOrNoneManagerModel.objects.create(my_field=True)

        queried_obj = ModelWithGetOrNoneManagerModel.objects.get_or_none(pk=new_obj.id)

        self.assertEqual(new_obj, queried_obj)

    def test_get_or_none_via_multiple_query_params(self):
        new_obj = ModelWithGetOrNoneManagerModel.objects.create(my_field=True)

        queried_obj = ModelWithGetOrNoneManagerModel.objects.get_or_none(pk=new_obj.id, my_field=True)

        self.assertEqual(new_obj, queried_obj)

    def test_get_or_none_no_results(self):
        ModelWithGetOrNoneManagerModel.objects.create(my_field=True)

        queried_obj = ModelWithGetOrNoneManagerModel.objects.get_or_none(my_field=False)

        self.assertIsNone(queried_obj)

    def test_get_or_none_multiple_results(self):
        ModelWithGetOrNoneManagerModel.objects.bulk_create(
            (ModelWithGetOrNoneManagerModel(my_field=True), ModelWithGetOrNoneManagerModel(my_field=True))
        )

        with self.assertRaisesMessage(
            ModelWithGetOrNoneManagerModel.MultipleObjectsReturned,
            "get() returned more than one ModelWithGetOrNoneManagerModel -- it returned 2!",
        ):
            ModelWithGetOrNoneManagerModel.objects.get_or_none(my_field=True)



from django.test import override_settings

from ambient_toolbox.static_role_permissions.settings import (
    get_static_role_permissions_enable_system_check,
    get_static_role_permissions_path,
)


@override_settings(STATIC_ROLE_PERMISSIONS_PATH="/path/to/permissions")
def test_get_static_role_permissions_path_is_set():
    assert get_static_role_permissions_path() == "/path/to/permissions"


def test_get_static_role_permissions_path_default_used():
    assert get_static_role_permissions_path() is None


@override_settings(STATIC_ROLE_PERMISSIONS_ENABLE_SYSTEM_CHECK=False)
def test_get_static_role_permissions_enable_system_check_is_set():
    assert get_static_role_permissions_enable_system_check() is False


def test_get_static_role_permissions_enable_system_check_default_used():
    assert get_static_role_permissions_enable_system_check() is True
